package com.ruoyi.system.service.impl;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.mapper.FcryMapper;
import com.ruoyi.system.mapper.FkqkMapper;
import com.ruoyi.system.mapper.GldryMapper;
import com.ruoyi.system.mapper.HscjMapper;
import io.netty.util.internal.ObjectUtil;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.math3.stat.descriptive.summary.Product;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.system.mapper.YxryMapper;
import com.ruoyi.system.service.IYxryService;

import javax.servlet.http.HttpServletResponse;

/**
 * 阳性人员Service业务层处理
 *
 * @author ruoyi
 * @date 2022-11-28
 */
@Service
public class YxryServiceImpl implements IYxryService {

    @Autowired
    private FcryMapper fcryMapper;

    @Autowired
    private YxryMapper yxryMapper;

    @Autowired
    private GldryMapper gldryMapper;

    @Autowired
    private HscjMapper hscjMapper;

    @Autowired
    private FkqkMapper fkqkMapper;

    /**
     * 查询阳性人员
     *
     * @param id 阳性人员主键
     * @return 阳性人员
     */
    @Override
    public Yxry selectYxryById(Long id) {
        return yxryMapper.selectYxryById(id);
    }

    /**
     * 查询阳性人员列表
     *
     * @param yxry 阳性人员
     * @return 阳性人员
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<Yxry> selectYxryList(Yxry yxry) {
        return yxryMapper.selectYxryList(yxry);
    }

    /**
     * 新增阳性人员
     *
     * @param yxry 阳性人员
     * @return 结果
     */
    @Override
    public int insertYxry(Yxry yxry) {
        String createBy = DataUtils.getValue(yxry.getArea(), yxry.getStreet(), yxry.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        if (CheckUtils.checkCard(yxry.getCardNo()))
            yxry.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
        yxry.setCreateBy(createBy);
        yxry.setCreateTime(DateUtils.getNowDate());
        return yxryMapper.insertYxry(yxry);
    }

    /**
     * 修改阳性人员
     *
     * @param yxry 阳性人员
     * @return 结果
     */
    @Override
    public int updateYxry(Yxry yxry) {
        String createBy = DataUtils.getValue(yxry.getArea(), yxry.getStreet(), yxry.getCommittee());
        if (createBy.equals("null"))
            throw new ServiceException("所选居住地、街道、社区不存在");
        if (CheckUtils.checkCard(yxry.getCardNo()))
            yxry.setCheckCard("正确");
        else
            throw new ServiceException("身份证格式不正确");
        yxry.setCreateBy(createBy);
        yxry.setUpdateTime(DateUtils.getNowDate());
        return yxryMapper.updateYxry(yxry);
    }

    /**
     * 批量删除阳性人员
     *
     * @param ids 需要删除的阳性人员主键
     * @return 结果
     */
    @Override
    public int deleteYxryByIds(Long[] ids) {
        return yxryMapper.deleteYxryByIds(ids);
    }

    /**
     * 删除阳性人员信息
     *
     * @param id 阳性人员主键
     * @return 结果
     */
    @Override
    public int deleteYxryById(Long id) {
        return yxryMapper.deleteYxryById(id);
    }

    /**
     * 导入数据
     *
     * @return 结果
     */
    @Override
    public void importList(List<Yxry> list, HttpServletResponse response) {
        if (StringUtils.isNull(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        List<Yxry> trueList = new ArrayList<>();
        List<Yxry> failureList = new ArrayList<>();
        List<Yxry> updateList = new ArrayList<>();
        for (Yxry zlqk : list) {
            if (StringUtils.isNotEmpty(zlqk.getCardNo()) && StringUtils.isNotEmpty(zlqk.getUserName())
                    && StringUtils.isNotEmpty(zlqk.getPhone()) && StringUtils.isNotEmpty(zlqk.getArea())) {
                String community = "";
                String committee = "";
                String street = "";
                if (StringUtils.isNotEmpty(zlqk.getCommunity())) { //如果小区不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("community");
                    List<String> values = communitys.stream().filter(l -> zlqk.getCommunity().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        community = values.get(0);
                    else {
                        failureList.add(zlqk);
                        continue;
                    }
                }
                if (StringUtils.isNotEmpty(zlqk.getCommittee())) { //如果委员会不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("committee");
                    List<String> values = communitys.stream().filter(l -> zlqk.getCommittee().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        committee = values.get(0);
                    else {
                        failureList.add(zlqk);
                        continue;
                    }
                }
                if (StringUtils.isNotEmpty(zlqk.getStreet())) { //如果街道不为空，去数据字典匹配value
                    List<SysDictData> communitys = DictUtils.getDictCache("street_town");
                    List<String> values = communitys.stream().filter(l -> zlqk.getStreet().equals(l.getDictLabel()))
                            .map(SysDictData::getDictValue).collect(Collectors.toList());
                    if (!values.isEmpty())
                        street = values.get(0);
                    else {
                        failureList.add(zlqk);
                        continue;
                    }
                }
                if ((StringUtils.isNotEmpty(community) && StringUtils.isEmpty(committee)) ||
                        (StringUtils.isNotEmpty(committee) && StringUtils.isEmpty(street))) {
                    failureList.add(zlqk);
                    continue;
                }
                //校验身份证是否重复
                List<Yxry> zlqks = yxryMapper.selectYxryByCard(zlqk.getCardNo());
                if (!zlqks.isEmpty()) {
                    zlqk.setCommunity(community);
                    zlqk.setCommittee(committee);
                    zlqk.setStreet(street);
                    updateList.add(zlqk);
                } else {
                    zlqks = list.stream().filter(l -> zlqk.getCardNo().equals(l.getCardNo())).collect(Collectors.toList());
                    if (zlqks.size() > 1)
                        failureList.add(zlqk);
                    else {
                        if (!CheckUtils.checkPhone(zlqk.getPhone())) {
                            failureList.add(zlqk);
                        } else {
                            if (CheckUtils.checkCard(zlqk.getCardNo())) {
                                zlqk.setCheckCard("正确");
                                zlqk.setCommunity(community);
                                zlqk.setCommittee(committee);
                                zlqk.setStreet(street);
                                trueList.add(zlqk);
                            } else {
                                zlqk.setCheckCard("不正确");
                                failureList.add(zlqk);
                            }
                        }
                    }
                }
            } else if (StringUtils.isEmpty(zlqk.getCardNo()) && StringUtils.isEmpty(zlqk.getUserName())
                    && StringUtils.isEmpty(zlqk.getPhone()) && StringUtils.isEmpty(zlqk.getStreet())
                    && StringUtils.isEmpty(zlqk.getArea()) && StringUtils.isEmpty(zlqk.getCommittee())) {
                continue;
            } else {
                failureList.add(zlqk);
            }
        }
        if (!trueList.isEmpty()) {
            for (Yxry zlqk : trueList) {
                zlqk.setCreateBy(DataUtils.getValue(zlqk.getArea(), zlqk.getStreet(), zlqk.getCommittee()));
                zlqk.setCreateTime(DateUtils.getNowDate());
            }
            yxryMapper.insertBatchYxry(trueList);
        }
        if (!updateList.isEmpty()) {
            for (Yxry zlqk : updateList) {
                zlqk.setCreateBy(DataUtils.getValue(zlqk.getArea(), zlqk.getStreet(), zlqk.getCommittee()));
                zlqk.setUpdateTime(DateUtils.getNowDate());
            }
            yxryMapper.updateBatchYxry(updateList);
        }
        if (!failureList.isEmpty()) {
            ExcelUtil<Yxry> util = new ExcelUtil<Yxry>(Yxry.class);
            util.exportExcel(response, failureList, "阳性人员数据");
//            return AjaxResult.error(111,"导出失败数据"+failureList.size()+"条");
        }
//            return AjaxResult.success("数据导入成功");
    }

    @Override
    public Map selectByCard(String card) {
        List<Yxry> zlqks = yxryMapper.selectYxryByCard(card);
        Map map = new HashMap();
        // 根据身份证号查询方舱人员信息
        Fcry fcry = new Fcry();
        fcry.setDelFlag("0");
        fcry.setCardNo(card);
        List<Fcry> fcries = fcryMapper.selectFcryList(fcry);
        // 根据身份证号查询隔离人员
        Gldry gldry = new Gldry();
        gldry.setDelFlag("0");
        gldry.setCardNo(card);
        List<Gldry> gldries = gldryMapper.selectGldryList(gldry);
        // 都找不到
        if (fcries.size() == 0 && gldries.size() == 0) {
            map.put("glPlace", "其他");
            map.put("fcName", null);
            map.put("glName", null);
        }
        List<Fcry> fcryList = fcries.stream().filter(f -> ObjectUtils.isEmpty(f.getZcTime())).collect(Collectors.toList());
        List<Gldry> gldryList = gldries.stream().filter(s -> ObjectUtils.isEmpty(s.getZcTime())).collect(Collectors.toList());
        // 都转出了
        if (fcryList.size() == 0 && gldryList.size() == 0) {
            map.put("glPlace", "其他");
            map.put("fcName", null);
            map.put("glName", null);
        } else if (fcryList.size() > 0 && gldryList.size() > 0) { // 都没转出
            map.put("glPlace", "其他");
            map.put("fcName", DictUtils.getDictLabel("shelters", fcryList.stream().map(Fcry::getFcPlace).collect(Collectors.joining(","))));
            map.put("glName", DictUtils.getDictLabel("isolation_point", gldryList.stream().map(Gldry::getGlPlace).collect(Collectors.joining(","))));
        } else if (fcryList.size() > 0) {
            map.put("glPlace", "方舱隔离");
            map.put("fcName", DictUtils.getDictLabel("shelters", fcryList.stream().map(Fcry::getFcPlace).collect(Collectors.joining(","))));
            map.put("glName", null);
        } else {
            map.put("glPlace", "隔离点隔离");
            map.put("fcName", null);
            map.put("glName", DictUtils.getDictLabel("isolation_point", gldryList.stream().map(Gldry::getGlPlace).collect(Collectors.joining(","))));
        }
        map.put("card", zlqks.isEmpty());
        return map;
    }

    @Override
    public Map statistics() {
        Map map = new HashMap();
        List<Object> todayData = new ArrayList<>();
        Map today = new HashMap();
        // 查询当天阳性人员数量
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String begin = sdf.format(TimeUtils.getStartTime());
        String end = sdf.format(TimeUtils.getEndTime());
        List<Yxry> yxryList = yxryMapper.statistics(begin, end);
        today.put("positiveNum", yxryList.size());
        // 涉及小区数量
        today.put("PositiveAreaNum", yxryList.stream().filter(y -> ObjectUtils.isNotEmpty(y.getCommunity()))
                .distinct().map(Yxry::getCommunity).collect(Collectors.toList()).size());
        // 涉及单元数量
        Map<String, List<Yxry>> prodMap = yxryList.stream().filter(y -> ObjectUtils.isNotEmpty(y.getCommunity())
                        && ObjectUtils.isNotEmpty(y.getBuilding()) && ObjectUtils.isNotEmpty(y.getUnit()))
                .collect(Collectors.groupingBy(item -> item.getCommunity() + "_" + item.getBuilding() + "_" + item.getUnit()));
        today.put("unitBuildingInvolvedNum", prodMap.keySet().size());
        // 查询当天采集异常数据(身份证不为空)
        List<Hscj> hscjList = hscjMapper.selectToday(begin, end);
        today.put("abnormalMixingNum", hscjList.stream().filter(cj -> ObjectUtils.isNotEmpty(cj.getCjgh())).distinct()
                .map(Hscj::getCjgh).collect(Collectors.toList()).size());
        today.put("abnormalMixingOfPipesNum", hscjList.stream().distinct().map(Hscj::getCardNo).collect(Collectors.toList()).size());
        // 查询封控数据
        LocalDate date = LocalDate.now();
        List<Fkqk> fkqkList = fkqkMapper.selectFkToday(date);
        // 获取真实时间是空的
        today.put("cellBlockingControlNum", fkqkList.stream().filter(f -> ObjectUtils.isEmpty(f.getRealTime())
                        && ObjectUtils.isNotEmpty(f.getCommunity()))
                .distinct().map(Fkqk::getCommunity).collect(Collectors.toList()).size());
        Map<String, List<Fkqk>> fkMap = fkqkList.stream().filter(f -> ObjectUtils.isEmpty(f.getRealTime())
                        && ObjectUtils.isNotEmpty(f.getFloor()) && ObjectUtils.isNotEmpty(f.getUnit()))
                .collect(Collectors.groupingBy(item -> item.getCommunity() + "_" + item.getFloor() + "_" + item.getUnit()));
        today.put("unitSealingControlNum", fkMap.keySet().size());
        Map<String, List<Fkqk>> jfMap = fkqkList.stream().filter(f -> ObjectUtils.isNotEmpty(f.getRealTime())
                        && ObjectUtils.isNotEmpty(f.getFloor()) && ObjectUtils.isNotEmpty(f.getUnit()))
                .collect(Collectors.groupingBy(item -> item.getCommunity() + "_" + item.getFloor() + "_" + item.getUnit()));
        today.put("unpackingUnitNum", jfMap.keySet().size());
        today.put("transferToShelterNum", fcryMapper.selectToday(date));
        today.put("transferToIsolationPointNum", gldryMapper.selectToday(date));
        todayData.add(today);
        map.put("todayData", todayData);
        List<Object> currentData = new ArrayList<>();
        Map curr = new HashMap();
        curr.put("isolationPointNum", gldryMapper.selectCurrent());
        curr.put("sheltersNum", fcryMapper.selectCurrent());
        List<Fkqk> fkqks = fkqkMapper.selectCurrent();
        List<String> j = fkqks.stream().distinct().map(Fkqk::getCommunity)
                .collect(Collectors.toList());
        System.out.println(j.size());
        curr.put("BlockedControlCellNum", fkqks.stream().distinct().map(Fkqk::getCommunity).collect(Collectors.toList()).size());
        Map<String, List<Fkqk>> fk = fkqks.stream()
                .collect(Collectors.groupingBy(item -> item.getCommunity() + "_" + item.getFloor() + "_" + item.getUnit()));
        curr.put("sealingControlUnitNum", fk.keySet().size());
        List<Yxry> yxries = yxryMapper.statistics(null, null);
        curr.put("positivePersonnelNum", yxries.size());
        currentData.add(curr);
        map.put("currentData", currentData);
        List<Object> CumulativeData = new ArrayList<>();
        Map cum = new HashMap();
        cum.put("CumulativePositivePerNum", yxryMapper.selectNum());
        List<Fkqk> fks = fkqkMapper.selectCumulativeData();
        cum.put("CumulativeSealingControlUnitNum", fks.size());
        cum.put("CumulativeBlockedControlCellNum", fks.stream().distinct().map(Fkqk::getCommunity)
                .collect(Collectors.toList()).size());
        cum.put("CumulativeUnpackingUnitNum", fks.stream().filter(f -> f.getRealTime()!=null)
                .distinct().map(Fkqk::getCommunity)
                .collect(Collectors.toList()).size());
        CumulativeData.add(cum);
        map.put("CumulativeData", CumulativeData);
        return map;
    }
}
